; 
    IDC_DEBUG_LIST equ 1111
; 

    ; --------------------------------------------------------------
    ; This extends the MASM32 dialog macros to do modeless dialogs.
    ; --------------------------------------------------------------

    CallModelessDialog MACRO Instance,Parent,DlgProc,lParamInit

      invoke CreateDialogIndirectParam,Instance,esi,Parent,
                                       ADDR DlgProc,lParamInit
      push eax                ;; preserve return value
      invoke GlobalFree,esi   ;; free memory
      pop eax                 ;; restore return value
      pop edi
      pop esi

    ENDM

; 

    .data

      hDebugList dd 0
      hDebugDlg  dd 0

    .code

; 

DebugDlgProc proc hWndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL rc:RECT

    Switch uMsg

      Case WM_INITDIALOG

        ;------------------------------------------------------
        ; Size the list box to fill the client area. Since the
        ; dialog is not user sizable, this sizing needs to be
        ; done only once.
        ;------------------------------------------------------

        invoke GetDlgItem, hWndDlg, IDC_DEBUG_LIST
        mov hDebugList, eax
        invoke GetClientRect, hWndDlg, ADDR rc
        invoke MoveWindow, hDebugList, 0, 0, rc.right, rc.bottom, TRUE

        return TRUE

      Case WM_COMMAND

        Switch wParam

          Case IDCANCEL

            ; ---------------------------------------------
            ; A modeless dialog is closed and destroyed in
            ; the same was as a normal application window.
            ; ---------------------------------------------

            invoke DestroyWindow, hWndDlg

        EndSw

      Case WM_DESTROY

        invoke PostQuitMessage, NULL

    EndSw

    return 0

DebugDlgProc endp

; 

DebugThreadProc proc lpParameter:LPVOID

    LOCAL msg:MSG

    ; ------------------------------------------------
    ; For a modeless dialog the WS_VISIBLE style must
    ; be specified for the dialog to be visible.
    ; ------------------------------------------------

    Dialog  "Debug", \                        ; caption
            "MS Sans Serif", 10, \            ; font,pointsize
            WS_OVERLAPPED or WS_SYSMENU or \  ; window style
            WS_MINIMIZEBOX or \
            WS_VISIBLE or DS_CENTER, \
            1, \                              ; number of controls
            0,0,120,90, \                     ; x, y, width, height
            1024                              ; memory buffer size

    ; ------------------------------------------------------------
    ; Instead of sizing the list box control to fill the client
    ; area by trial and error, specify a width and height of zero
    ; and leave the actual sizing for the WM_INITDIALOG handler.
    ;
    ; The LBS_NOINTEGRALHEIGHT is necessary so the list box can
    ; fill the client area of the window even if the height of
    ; the client area is not a integral multiple of the list
    ; item height.
    ; ------------------------------------------------------------

    DlgList WS_VSCROLL or LBS_NOINTEGRALHEIGHT,0,0,0,0,IDC_DEBUG_LIST

    invoke GetModuleHandle, NULL

    ; ----------------------------------------------------------
    ; We need a modeless dialog instead of a modal dialog so it
    ; will not retain the applications input focus while it is
    ; open, so we can continue working with the parent window.
    ;
    ; Use the handle pointed to by the lpParameter parameter as
    ; the handle of the parent window.
    ; ----------------------------------------------------------

    mov edx, lpParameter
    mov edx, [edx]
    CallModelessDialog eax, \           ; instance handle
                       edx, \           ; parent window handle
                       DebugDlgProc, \  ; dialog window procedure
                       NULL             ; optional argument

    mov hDebugDlg, eax

    ; -----------------------------------------
    ; Unlike a modal dialog, a modeless dialog
    ; must have a message loop.
    ; -----------------------------------------

  msgLoop:

    invoke GetMessage, ADDR msg, 0, 0, 0
    .IF (eax != 0)
      invoke IsDialogMessage, hDebugDlg, ADDR msg
      jmp msgLoop
    .ENDIF

    ; -------------------------------------------------------
    ; Zero hDebugDlg so the parent can use it to determine
    ; whether or not the debug window is open, and hDebugList
    ; so the CreateDebugWindow procedure will know to wait
    ; for the list box to be created.
    ; -------------------------------------------------------

    mov hDebugDlg, 0
    mov hDebugList, 0

    ; -------------------------------------------------------
    ; Exiting from the thread procedure will end the thread.
    ; -------------------------------------------------------

    ret

DebugThreadProc endp

; 

CreateDebugWindow proc hParent:HWND

  LOCAL threadID:DWORD

  ; --------------------------------------------------------
  ; Create a separate thread for the debug dialog (window).
  ; --------------------------------------------------------

  invoke CreateThread, NULL,
                       0,
                       ADDR DebugThreadProc,
                       ADDR hParent,
                       0,
                       ADDR threadID

  ; --------------------------------------------------------
  ; Wait until the list box handle is set before returning.
  ; --------------------------------------------------------

  .WHILE hDebugList == 0
  .ENDW

  ret

CreateDebugWindow endp

; 